write-b -- ------------------------------------------ --;
write   -- Les requtes du chapitre 9                 --;
write   -- Les dclencheurs                           --;
write   -- CLICOM avec redondances et journalisation  --;
write-a -- ------------------------------------------ --;

-- ------------------------------------- --
--  Application CLICOM avec redondances  --
-- ------------------------------------- --

-- REMARQUE : Pas de contrle d'accs en SQLite; donc pas de 
-- possibilit d'interdire la MAJ directe des colonnes calcules.

-- REMARQUE : En SQLite, un trigger qui accde  une table temporaire  
-- doit aussi tre temporaire (tous les "temporary" sont stocks
-- dans une BD nomme "temp"). Autre solution: create + drop JOURNAL.

set ; = ;;

createOrReplaceDB CLICOM-Redondance.db;

create table CLIENT (
        NCLI      char(10)    not null primary key,
        NOM       char(32)    not null,
        ADRESSE   char(60)    not null,
        LOCALITE  char(30)    not null,
        CAT       char(2),
        COMPTE    decimal(9,2)not null,
        TOTAL_CLI decimal(8)  not null default 0);

create table PRODUIT (
        NPRO      char(15)    not null primary key,
        LIBELLE   char(60)    not null,
        PRIX      decimal(6)  not null,
        QSTOCK    decimal(8)  not null)

create table COMMANDE (
        NCOM      char(12)    not null primary key,
        NCLI      char(10)    not null,
        DATECOM   date        not null,
        TOTAL_COM decimal(8)  not null default 0,
        foreign key (NCLI) references CLIENT
		on delete no action on update cascade);

create table DETAIL (
        NCOM      char(12)    not null,
        NPRO      char(15)    not null,
        QCOM      decimal(8)  not null,
        MONTANT   decimal(8)  not null default 0,
        primary key (NCOM,NPRO),
        foreign key (NCOM) references COMMANDE
		on delete cascade on update cascade,
        foreign key (NPRO) references PRODUIT
		on delete no action on update cascade);

create temp table JOURNAL (
        LogID      integer not null primary key autoincrement,
        DateTime   char(23) not null,
        Operation  char(32),
        Object varchar(64),
        Arguments  varchar(256));


-- ------------------------------------- --
-- ACTIONS RELATIVES A LA TABLE PRODUIT  --
-- ------------------------------------- --

-- INSERT into PRODUIT --

create temporary trigger TRG_PRO_INS
after insert on PRODUIT
for each row
begin
  insert into JOURNAL(DateTime,Operation,Object,Arguments)
         values(current_timestamp_full(),'insert','PRODUIT','NPRO = '||new.NPRO)$;$ 
end;

-- DELETE from PRODUIT --

create temporary trigger TRG_PRO_DEL
after delete on PRODUIT
for each row
begin
  insert into JOURNAL(DateTime,Operation,Object,Arguments)
         values(current_timestamp_full(),'delete','PRODUIT','NPRO = '||old.NPRO)$;$ 
end;

-- UPDATE PRODUIT (PRIX) --

create temporary trigger TRG_PRO_UPD_PRIX
after update of PRIX on PRODUIT
for each row
begin
  insert into JOURNAL(DateTime,Operation,Object,Arguments)
         values(current_timestamp_full(),'update','PRODUIT',
                'NPRO = '||old.NPRO||'; PRIX = '||old.PRIX||' > '||new.PRIX)$;$ 
  update DETAIL 
  set    MONTANT = QCOM * new.PRIX
  where  NPRO = new.NPRO$;$ 
end;


-- ------------------------------------- --
-- ACTIONS RELATIVES A LA TABLE DETAIL   --
-- ------------------------------------- --

-- INSERT into DETAIL --

create temporary trigger TRG_DET_INS
after insert on DETAIL
for each row
begin
  insert into JOURNAL(DateTime,Operation,Object,Arguments)
         values(current_timestamp_full(),'insert','DETAIL',
                'NCOM = '||new.NCOM||' & NPRO = '||new.NPRO)$;$ 
  update DETAIL 
  set    MONTANT = new.QCOM * (select PRIX
                               from   PRODUIT
                               where  NPRO = new.NPRO)
  where  NCOM = new.NCOM and NPRO = new.NPRO$;$
end;


-- DELETE from DETAIL --

create temporary trigger TRG_DET_DEL
after delete on DETAIL
for each row
begin
  insert into JOURNAL(DateTime,Operation,Object,Arguments)
         values(current_timestamp_full(),'delete','DETAIL',
                'NCOM = '||old.NCOM||' & NPRO = '||old.NPRO)$;$ 
  update COMMANDE 
  set    TOTAL_COM = coalesce((select sum(MONTANT)
                               from   DETAIL
                               where  NCOM = old.NCOM),0)
  where  NCOM = old.NCOM$;$ 
end;


-- UPDATE DETAIL (NCOM,NPRO,QCOM,MONTANT) --

create temporary trigger TRG_DET_UPD_NCOM
after update of NCOM on DETAIL
for each row
begin
  insert into JOURNAL(DateTime,Operation,Object,Arguments)
         values(current_timestamp_full(),'update','DETAIL',
                'NCOM&NPRO = '||new.NCOM||'&'||new.NPRO||'; NCOM = '||old.NCOM||' > '||new.NCOM)$;$ 
  update COMMANDE 
  set    TOTAL_COM = coalesce((select sum(MONTANT)
                               from   DETAIL
                               where  NCOM = old.NCOM),0)
  where  NCOM = old.NCOM$;$ 
  update COMMANDE 
  set    TOTAL_COM = (select sum(MONTANT)
                      from   DETAIL
                      where  NCOM = new.NCOM)
  where  NCOM = new.NCOM$;$ 
end;

create temporary trigger TRG_DET_UPD_NPRO
after update of NPRO on DETAIL
for each row
begin
  insert into JOURNAL(DateTime,Operation,Object,Arguments)
         values(current_timestamp_full(),'update','DETAIL',
                'NPRO = '||old.NPRO||' > '||new.NPRO)$;$ 
  update DETAIL 
  set    MONTANT = new.QCOM * (select PRIX
                               from   PRODUIT
                               where  NPRO = new.NPRO)
  where  NCOM = new.NCOM and NPRO = new.NPRO$;$ 
end;

create temporary trigger TRG_DET_UPD_QCOM
after update of QCOM on DETAIL
for each row
begin
  insert into JOURNAL(DateTime,Operation,Object,Arguments)
         values(current_timestamp_full(),'update','DETAIL',
                'QCOM = '||old.QCOM||' > '||new.QCOM)$;$ 
  update DETAIL 
  set    MONTANT = new.QCOM * (select PRIX
                               from   PRODUIT
                               where  NPRO = new.NPRO)
  where  NCOM = new.NCOM and NPRO = new.NPRO$;$ 
end;

create temporary trigger TRG_DET_UPD_MONTANT
after update of MONTANT on DETAIL
for each row
begin
  insert into JOURNAL(DateTime,Operation,Object,Arguments)
         values(current_timestamp_full(),'update','DETAIL',
                'MONTANT = '||old.MONTANT||' > '||new.MONTANT)$;$ 
  update COMMANDE 
  set    TOTAL_COM = (select sum(MONTANT)
                      from   DETAIL
                      where  NCOM = new.NCOM)
  where  NCOM = new.NCOM$;$ 
end;


-- ------------------------------------- --
-- ACTIONS RELATIVES A LA TABLE COMMANDE --
-- ------------------------------------- --

-- INSERT into COMMANDE --

create temporary trigger TRG_COM_INS
after insert on COMMANDE
for each row
begin
  insert into JOURNAL(DateTime,Operation,Object,Arguments)
         values(current_timestamp_full(),'insert','COMMANDE','NCOM = '||new.NCOM)$;$ 
end;

-- DELETE from COMMANDE --

create temporary trigger TRG_COM_DEL
after delete on COMMANDE
for each row
begin
  insert into JOURNAL(DateTime,Operation,Object,Arguments)
         values(current_timestamp_full(),'delete','COMMANDE','NCOM = '||old.NCOM)$;$ 
  update CLIENT 
  set    TOTAL_CLI = coalesce((select sum(TOTAL_COM)
                               from   COMMANDE
                               where  NCLI = old.NCLI),0)
  where  NCLI = old.NCLI$;$ 
end;


-- UPDATE COMMANDE (NCLI,TOTAL_COM) --

create temporary trigger TRG_COM_UPD_NCLI
after update of NCLI on COMMANDE
for each row
begin
  insert into JOURNAL(DateTime,Operation,Object,Arguments)
         values(current_timestamp_full(),'update','COMMANDE',
                'NCLI = '||old.NCLI||' > '||new.NCLI)$;$ 
  update CLIENT 
  set    TOTAL_CLI = coalesce((select sum(TOTAL_COM)
                               from   COMMANDE
                               where  NCLI = old.NCLI),0)
  where  NCLI = old.NCLI$;$ 
  update CLIENT 
  set    TOTAL_CLI = (select sum(TOTAL_COM)
                      from   COMMANDE
                      where  NCLI = new.NCLI)
  where  NCLI = new.NCLI$;$ 
end;

create temporary trigger TRG_COM_UPD_TOTAL_COM
after update of TOTAL_COM on COMMANDE
for each row
begin
  insert into JOURNAL(DateTime,Operation,Object,Arguments)
         values(current_timestamp_full(),'update','COMMANDE',
                'TOTAL_COM = '||old.TOTAL_COM||' > '||new.TOTAL_COM)$;$ 
  update CLIENT 
  set    TOTAL_CLI = (select sum(TOTAL_COM)
                      from   COMMANDE
                      where  NCLI = new.NCLI)
  where  NCLI = new.NCLI$;$ 
end;

-- ------------------------------------- --
-- ACTIONS RELATIVES A LA TABLE CLIENT   --
-- ------------------------------------- --

-- INSERT into CLIENT --

create temporary trigger TRG_CLI_INS
after insert on CLIENT
for each row
begin
  insert into JOURNAL(DateTime,Operation,Object,Arguments)
         values(current_timestamp_full(),'insert','CLIENT','NCLI = '||new.NCLI)$;$ 
end;

-- DELETE from CLIENT --

create temporary trigger TRG_CLI_DEL
after delete on CLIENT
for each row
begin
  insert into JOURNAL(DateTime,Operation,Object,Arguments)
         values(current_timestamp_full(),'delete','CLIENT','NCLI = '||old.NCLI)$;$ 
end;

-- UPDATE CLIENT.TOTAL_CLI    --

create temporary trigger TRG_CLI_UPD_TOTAL_CLI
after update of TOTAL_CLI on CLIENT
for each row
begin
  insert into JOURNAL(DateTime,Operation,Object,Arguments)
         values(current_timestamp_full(),'update','CLIENT',
                'TOTAL_CLI = '||old.TOTAL_CLI||' > '||new.TOTAL_CLI)$;$ 
end;


set Temps-initial = $time$;

-- ------------------------------------- --
-- CHARGEMENT DES DONNEES BRUTES         --
-- ------------------------------------- --

insert into CLIENT(NCLI,NOM,ADRESSE,LOCALITE,CAT,COMPTE) values
    ('B112','HANSENNE','23, r. Dumont','Poitiers','C1',1250);
insert into CLIENT(NCLI,NOM,ADRESSE,LOCALITE,CAT,COMPTE) values
    ('C123','MERCIER','25, r. Lematre','Namur','C1',-2300);
insert into CLIENT(NCLI,NOM,ADRESSE,LOCALITE,CAT,COMPTE) values
    ('B332','MONTI','112, r. Neuve','Genve','B2',0);
insert into CLIENT(NCLI,NOM,ADRESSE,LOCALITE,CAT,COMPTE) values
    ('F010','TOUSSAINT','5, r. Godefroid','Poitiers','C1',0);
insert into CLIENT(NCLI,NOM,ADRESSE,LOCALITE,CAT,COMPTE) values
    ('K111','VANBIST','180, r. Florimont','Lille','B1',720);
insert into CLIENT(NCLI,NOM,ADRESSE,LOCALITE,CAT,COMPTE) values
    ('S127','VANDERKA','3, av. des Roses','Namur','C1',-4580);
insert into CLIENT(NCLI,NOM,ADRESSE,LOCALITE,CAT,COMPTE) values
    ('B512','GILLET','14, r. de l''Et','Toulouse','B1',-8700);
insert into CLIENT(NCLI,NOM,ADRESSE,LOCALITE,CAT,COMPTE) values
    ('B062','GOFFIN','72, r. de la Gare','Namur','B2',-3200);
insert into CLIENT(NCLI,NOM,ADRESSE,LOCALITE,CAT,COMPTE) values
    ('C400','FERARD','65, r. du Tertre','Poitiers','B2',350);
insert into CLIENT(NCLI,NOM,ADRESSE,LOCALITE,CAT,COMPTE) values
    ('C003','AVRON','8, ch. de la Cure','Toulouse','B1',-1700);
insert into CLIENT(NCLI,NOM,ADRESSE,LOCALITE,CAT,COMPTE) values
    ('K729','NEUMAN','40, r. Bransart','Toulouse',null,0);
insert into CLIENT(NCLI,NOM,ADRESSE,LOCALITE,CAT,COMPTE) values
    ('F011','PONCELET','17, Cls des Erables','Toulouse','B2',0);
insert into CLIENT(NCLI,NOM,ADRESSE,LOCALITE,CAT,COMPTE) values
    ('L422','FRANCK','60, r. de Wpion','Namur','C1',0);
insert into CLIENT(NCLI,NOM,ADRESSE,LOCALITE,CAT,COMPTE) values
    ('S712','GUILLAUME','14a, ch. des Roses','Paris','B1',0);
insert into CLIENT(NCLI,NOM,ADRESSE,LOCALITE,CAT,COMPTE) values
    ('D063','MERCIER','201, bvd du Nord','Toulouse',null,-2250);
insert into CLIENT(NCLI,NOM,ADRESSE,LOCALITE,CAT,COMPTE) values
    ('F400','JACOB','78, ch. du Moulin','Bruxelles','C2',0);

insert into PRODUIT values ('CS262','CHEV. SAPIN 200x6x2',75,45);
insert into PRODUIT values ('CS264','CHEV. SAPIN 200x6x4',120,2690);
insert into PRODUIT values ('CS464','CHEV. SAPIN 400x6x4',220,450);
insert into PRODUIT values ('PA45' ,'POINTE ACIER 45 (1K)',105,580);
insert into PRODUIT values ('PA60' ,'POINTE ACIER 60 (1K)',95,134);
insert into PRODUIT values ('PH222','PL. HETRE 200x20x2',230,782);
insert into PRODUIT values ('PS222','PL. SAPIN 200x20x2',185,1220);

insert into COMMANDE(NCOM,NCLI,DATECOM) values ('30178','K111','2015-12-21');
insert into COMMANDE(NCOM,NCLI,DATECOM) values ('30179','C400','2015-12-22');
insert into COMMANDE(NCOM,NCLI,DATECOM) values ('30182','S127','2015-12-23');
insert into COMMANDE(NCOM,NCLI,DATECOM) values ('30184','C400','2015-12-23');
insert into COMMANDE(NCOM,NCLI,DATECOM) values ('30185','F011','2016-01-02');
insert into COMMANDE(NCOM,NCLI,DATECOM) values ('30186','C400','2016-01-02');
insert into COMMANDE(NCOM,NCLI,DATECOM) values ('30188','B512','2016-01-03');

insert into DETAIL(NCOM,NPRO,QCOM) values ('30178','CS464',25);
insert into DETAIL(NCOM,NPRO,QCOM) values ('30179','PA60',20);
insert into DETAIL(NCOM,NPRO,QCOM) values ('30179','CS262',60);
insert into DETAIL(NCOM,NPRO,QCOM) values ('30182','PA60',30);
insert into DETAIL(NCOM,NPRO,QCOM) values ('30184','CS464',120);
insert into DETAIL(NCOM,NPRO,QCOM) values ('30184','PA45',20);
insert into DETAIL(NCOM,NPRO,QCOM) values ('30185','PA60',15);
insert into DETAIL(NCOM,NPRO,QCOM) values ('30185','PS222',600);
insert into DETAIL(NCOM,NPRO,QCOM) values ('30185','CS464',260);
insert into DETAIL(NCOM,NPRO,QCOM) values ('30186','PA45',3);
insert into DETAIL(NCOM,NPRO,QCOM) values ('30188','PA60',70);
insert into DETAIL(NCOM,NPRO,QCOM) values ('30188','PH222',92);
insert into DETAIL(NCOM,NPRO,QCOM) values ('30188','CS464',180);
insert into DETAIL(NCOM,NPRO,QCOM) values ('30188','PA45',22);

commitDB;

set Temps-intermdiaire = $time$;

-- ---------------------------------- --
-- Etat initial de la BD              --
-- ---------------------------------- --

write-b table CLIENT ;
select * from CLIENT order by NCLI;

write-b table PRODUIT;
select * from PRODUIT order by NPRO;

write-b table COMMANDE;
select * from COMMANDE order by NCOM;

write-b DETAIL table;
select * from DETAIL order by NCOM, NPRO;

-- ---------------------------------- --
-- UPDATE PRIX                        --
-- ---------------------------------- --

write-b -- update PRODUIT set PRIX = -100 where NPRO = 'PA45' --;
update PRODUIT set PRIX = -20 where NPRO = 'PA45';

-- ---------------------------------- --
-- UPDATE QCOM                        --
-- ---------------------------------- --

write-b -- update DETAIL set QCOM = -100 where NCOM = '30188' and NPRO = 'PA45' --;
update DETAIL set QCOM = -100 where NCOM = '30188' and NPRO = 'PA45';

-- ---------------------------------- --
-- UPDATE DETAIL.NPRO                 --
-- ---------------------------------- --

write-b -- update DETAIL set NPRO = 'CS262' where NPRO = 'PA45' --;
update DETAIL set NPRO = 'CS262' where NPRO = 'PA45';

-- ---------------------------------- --
-- UPDATE DETAIL.NPRO,NCOM,QCOM       --
-- ---------------------------------- --

write-b -- update DETAIL;
write   -- set NCOM = '30186',;
write   --     NPRO = 'PA60',;
write   --     QCOM = 68;
write   -- where NCOM = '30185' and NPRO = 'PS222';

update DETAIL
set NCOM = '30186',
    NPRO = 'PA60',
    QCOM = 68
where NCOM = '30185' and NPRO = 'PS222';

-- ---------------------------------- --
-- UPDATE COMMANDE.NCLI               --
-- ---------------------------------- --

write-b -- update COMMANDE set NCLI = 'C400' where NCLI = 'B512' --;
update COMMANDE set NCLI = 'C400' where NCLI = 'B512';

set Temps-final = $time$;

-- ---------------------------------- --
-- DELETE from DETAIL                 --
-- ---------------------------------- --

delete from COMMANDE where NCOM = '30188';


-- ---------------------------------- --
-- Etat final de la BD                --
-- ---------------------------------- --

write-b table CLIENT;
select * from CLIENT order by NCLI;

write-b table PRODUIT;
select * from PRODUIT order by NPRO;

write-b table COMMANDE;
select * from COMMANDE order by NCOM;

write-b table DETAIL;
select * from DETAIL order by NCOM, NPRO;

write-b table JOURNAL;
select * from JOURNAL;


write Temps-initial =       $Temps-initial$;
write Temps-intermdiaire = $Temps-intermdiaire$;
write Temps-final =         $Temps-final$;

closeDB;

write;
