write-b -- -------------------------- --;
write   -- Les requtes du chapitre 9 --;
write   -- Les requtes rcursives    --;
write   --        COMPOSITION         --;
write-a -- -------------------------- --;

createOrReplaceDB NOMENCLATURE.db;

create table PRODUIT
  (NPRO      char(10) not null primary key,
   LIBELLE   char(8)  not null,
   PRIX_U    decimal(5,1));

create table COMPOSITION
  (COMPOSE   char(10) not null,
   COMPOSANT char(10) not null,
   QTE       integer  not null,
   primary key (COMPOSE,COMPOSANT),
   foreign key (COMPOSE) references PRODUIT
           on delete cascade
           on update cascade,
   foreign key (COMPOSANT) references PRODUIT
           on delete cascade
           on update cascade);

create view PRODUIT_FINI
as select NPRO,LIBELLE,PRIX_U 
   from   PRODUIT
   where  NPRO not in (select COMPOSANT from COMPOSITION);

create view MATIERE_PREMIERE
as select NPRO,LIBELLE,PRIX_U 
   from   PRODUIT
   where  NPRO not in (select COMPOSE from COMPOSITION);

-- DATA LOADING 

insert into PRODUIT values ('p01','A-200',null);
insert into PRODUIT values ('p02','A-056',null);
insert into PRODUIT values ('p03','B-661',null);
insert into PRODUIT values ('p04','B-122',null);
insert into PRODUIT values ('p05','B-326',null);
insert into PRODUIT values ('p06','D-822',3.5);
insert into PRODUIT values ('p07','D-507',8.0);
insert into PRODUIT values ('p08','G-993',5.0);
insert into PRODUIT values ('p09','F-016',null);
insert into PRODUIT values ('p10','J-500',null);
insert into PRODUIT values ('p11','J-544',0.5);
insert into PRODUIT values ('p12','L-009',1.7);

insert into COMPOSITION values ('p01','p02',2);
insert into COMPOSITION values ('p01','p03',1);
insert into COMPOSITION values ('p01','p04',2);
insert into COMPOSITION values ('p02','p07',8);
insert into COMPOSITION values ('p02','p08',2);
insert into COMPOSITION values ('p03','p08',5);
insert into COMPOSITION values ('p04','p08',4);
insert into COMPOSITION values ('p04','p09',5);
insert into COMPOSITION values ('p04','p10',5);
insert into COMPOSITION values ('p05','p04',2);
insert into COMPOSITION values ('p05','p06',7);
insert into COMPOSITION values ('p09','p11',2);
insert into COMPOSITION values ('p10','p11',4);
insert into COMPOSITION values ('p10','p12',3);

write-b -- PRODUITS --;
select * from PRODUIT;

write-b -- COMPOSITION --;
select * from COMPOSITION;

write-b -- Produits finis --;
select * from PRODUIT_FINI;

write-b -- Matires premires --;
select * from MATIERE_PREMIERE;

write-b -- Liste des compositions de tous les composs --;
with recursive CHAINE(Produit,Compo,Qte) as
    (select NPRO,NPRO,1 from PRODUIT 
       union all
     select Ch.Produit,Co.COMPOSANT,Ch.Qte * Co.QTE
     from   CHAINE Ch, COMPOSITION Co
     where  Ch.Compo = Co.COMPOSE
     )
select * from CHAINE where Produit <> Compo order by Produit,Compo;

write-b -- Consommation en matires premires --;
with recursive CHAINE(Produit,Compo,Qte) as
    (select NPRO,NPRO,1 from PRODUIT 
       union all
     select Ch.Produit,Co.COMPOSANT,Ch.Qte * Co.QTE
     from   CHAINE Ch, COMPOSITION Co
     where  Ch.Compo = Co.COMPOSE
     )
select Produit,Compo as "Mat. Premiere",sum(QTE) as Quantite
from   CHAINE
where  Compo in (select NPRO from MATIERE_PREMIERE)
and    Produit <> Compo
group by Produit,Compo
order by Produit,Compo;

write-b -- Cot des produits finis en matires premires --;
with recursive CHAINE(Produit,Compo,Qte) as
    (select NPRO,NPRO,1 from PRODUIT 
       union all
     select Ch.Produit,Co.COMPOSANT,Ch.Qte * Co.QTE
     from   CHAINE Ch, COMPOSITION Co
     where  Ch.Compo = Co.COMPOSE
     )
select Produit,sum(QTE*PRIX_U) as Cout
from   CHAINE Ch, PRODUIT Pro
where  Produit in (select NPRO from PRODUIT_FINI)
and    Compo   in (select NPRO from MATIERE_PREMIERE)
and    Compo = Pro.NPRO
and    Produit <> Compo
group by Produit;

-- Vrifier que toutes les matires premires ont un prix  l'unit.
extract N = select count(*) from MATIERE_PREMIERE
            where  PRIX_U is null;

if ($N$ > 0);
  write-b Donnes incompltes. Certaines matires premires nont pas de prix unitaire.;
  write-a La table PRODUIT ne peut tre mise  jour.;
else;
  with recursive CHAINE(Produit,Compo,Qte) as
      (select NPRO,NPRO,1 from PRODUIT
         union all
       select Ch.Produit, Co.COMPOSANT, Ch.Qte*Co.QTE
       from   CHAINE Ch, COMPOSITION Co
       where  Ch.Compo = Co.COMPOSE
       )
  update PRODUIT
  set    PRIX_U = (select sum(Qte*PRIX_U)
                   from   CHAINE, PRODUIT Pr
                   where  Compo = Pr.NPRO
                   and    Compo in (select NPRO
                                    from MATIERE_PREMIERE)
                   and    Produit = PRODUIT.NPRO
                   group by Produit)
  where  NPRO in (select COMPOSE from COMPOSITION);
endif;

select * from PRODUIT;

closeDB;


