-- ----------------------------------- --
-- Bases de donnes non relationnelles --
-- Le modle Attribut-valeur V2        --
-- ----------------------------------- --

openDB CLICOM.db;

create temporary table CLICOM(
   TypeE     varchar(32)   not null,
   Entite    varchar(32)   not null,
   Attribut  varchar(32)   not null,
   Valeur    varchar(256),
   primary key (TypeE,Entite,Attribut));


-- ------------------------------- --
-- index des identifiants par type --
-- ------------------------------- --

create unique index CLI_ID on CLICOM(Valeur)
       where TypeE = 'CLIENT' and Attribut = 'NCLI';
create unique index PRO_ID on CLICOM(Valeur)
       where TypeE = 'PRODUIT' and Attribut = 'NPRO';
create unique index COM_ID on CLICOM(Valeur)
       where TypeE = 'COMMANDE' and Attribut = 'NCOM';
-- L'unicit selon l'identifiant composite de DETAIL ne peut s'exprimer
-- par un index unique. Il faudra avoir recours  un dclencheur

-- Les cls trangres peuvent s'exprimer de deux manires : 
-- - soit via les valeurs des attributs, comme pour les tables sources
-- - soit via les valeurs de la colonne Entite,  la manire
--   des schmas relationnels objet
-- Dans ce dernier cas, les cls trangres seront reprsentes dans
-- des tables spcifiques.

-- ---------------------------------- --
-- Vue exprimant les donnes sources --
-- ---------------------------------- --

create temporary view V_CLIENT as
select 
max(case when Attribut = 'NCLI'     then Valeur end) as NCLI,
max(case when Attribut = 'NOM'      then Valeur end) as NOM,
max(case when Attribut = 'ADRESSE'  then Valeur end) as ADRESSE,
max(case when Attribut = 'LOCALITE' then Valeur end) as LOCALITE,
max(case when Attribut = 'CAT'      then Valeur end) as CAT,
max(case when Attribut = 'COMPTE'   then cast(Valeur as real) else -999999.9 end) as COMPTE
from  CLICOM
where TypeE = 'CLIENT'
group by Entite;

-- ---------------------- --
-- Chargement des donnes --
-- ---------------------- --

insert into CLICOM (TypeE,Entite,Attribut,Valeur) 
       select 'CLIENT',NCLI,'NCLI',NCLI from CLIENT;
insert into CLICOM (TypeE,Entite,Attribut,Valeur) 
       select 'CLIENT',NCLI,'NOM',NOM from CLIENT;
insert into CLICOM (TypeE,Entite,Attribut,Valeur) 
       select 'CLIENT',NCLI,'ADRESSE',ADRESSE from CLIENT;
insert into CLICOM (TypeE,Entite,Attribut,Valeur) 
       select 'CLIENT',NCLI,'LOCALITE',LOCALITE from CLIENT;
insert into CLICOM (TypeE,Entite,Attribut,Valeur) 
       select 'CLIENT',NCLI,'CAT',CAT from CLIENT;
insert into CLICOM (TypeE,Entite,Attribut,Valeur) 
       select 'CLIENT',NCLI,'COMPTE',COMPTE from CLIENT;

insert into CLICOM (TypeE,Entite,Attribut,Valeur) 
       select 'PRODUIT',NPRO,'NPRO',NPRO from PRODUIT;
insert into CLICOM (TypeE,Entite,Attribut,Valeur) 
       select 'PRODUIT',NPRO,'LIBELLE',LIBELLE from PRODUIT;
insert into CLICOM (TypeE,Entite,Attribut,Valeur) 
       select 'PRODUIT',NPRO,'PRIX',PRIX from PRODUIT;
insert into CLICOM (TypeE,Entite,Attribut,Valeur) 
       select 'PRODUIT',NPRO,'QSTOCK',QSTOCK from PRODUIT;

insert into CLICOM (TypeE,Entite,Attribut,Valeur) 
       select 'COMMANDE',NCOM,'NCOM',NCOM from COMMANDE;
insert into CLICOM (TypeE,Entite,Attribut,Valeur) 
       select 'COMMANDE',NCOM,'NCLI',NCLI from COMMANDE;
insert into CLICOM (TypeE,Entite,Attribut,Valeur) 
       select 'COMMANDE',NCOM,'DATECOM',DATECOM from COMMANDE;

insert into CLICOM (TypeE,Entite,Attribut,Valeur) 
       select 'DETAIL',NCOM||'-'||NPRO,'NCOM',NCOM from DETAIL;
insert into CLICOM (TypeE,Entite,Attribut,Valeur) 
       select 'DETAIL',NCOM||'-'||NPRO,'NPRO',NPRO from DETAIL;
insert into CLICOM (TypeE,Entite,Attribut,Valeur) 
       select 'DETAIL',NCOM||'-'||NPRO,'QCOM',QCOM from DETAIL;

write-b Table CLICOM ;
select * from CLICOM order by TypeE,Entite;

write-b Table source CLIENT reconstitue;
select * from V_CLIENT;

closeDB;

stop;
