-- ----------------------------------- --
-- Bases de donnes non relationnelles --
-- Le modle de graphes                --
-- ----------------------------------- --

openDB CLICOM.db;

create temporary table ENTITE(
   Entite    integer     not null primary key,
   TypeE     varchar(32) not null);

create temporary table ASSOCIATION(
   Source    integer not null,
   Cible     integer not null,
   TypeA     varchar(32) not null,
   primary key (TypeA,Source,Cible));

create temporary table ATTRIBUT(
   Entite    integer     not null,
   Attribut  varchar(32) not null,
   Valeur    varchar(256),
   primary key (Entite,Attribut));

create temporary table IDENTITE(
   Entite    integer     not null,
   TypeE     varchar(32) not null,
   EID       varchar(32) not null,
   primary key (TypeE,EID),
   unique (Entite));

-- ---------------------- --
-- Chargement des donnes --
-- ---------------------- --

extract entite = select coalesce(max(Entite),0) from ENTITE;

for cli,nom,adr,loc,cat,cpt = [select * from CLIENT];
  compute entite = $entite$ + 1;

  insert into ENTITE   values($entite$,'CLIENT');
  insert into IDENTITE values($entite$,'CLIENT','$cli$');

  function nom = LStr:SQLquote2 {$nom$};
  function adr = LStr:SQLquote2 {$adr$};
  function loc = LStr:SQLquote2 {$loc$};

  insert into ATTRIBUT values($entite$,'NCLI','$cli$');
  insert into ATTRIBUT values($entite$,'NOM','$nom$');
  insert into ATTRIBUT values($entite$,'ADRESSE','$adr$');
  insert into ATTRIBUT values($entite$,'LOCALITE','$loc$');
  insert into ATTRIBUT values($entite$,'CAT',case when '$cat$' = '' 
                                             then null else '$cat$'
                                             end);
  insert into ATTRIBUT values($entite$,'COMPTE',$cpt$);
endfor;

extract entite = select coalesce(max(Entite),0) from ENTITE;

for pro,lib,pri,qst = [select * from PRODUIT];
  compute entite = $entite$ + 1;

  insert into ENTITE values($entite$,'PRODUIT');
  insert into IDENTITE values($entite$,'PRODUIT','$pro$');

  function lib = LStr:SQLquote2 {$lib$};

  insert into ATTRIBUT values($entite$,'NPRO','$pro$');
  insert into ATTRIBUT values($entite$,'LIBELLE','$lib$');
  insert into ATTRIBUT values($entite$,'PRIX',$pri$);
  insert into ATTRIBUT values($entite$,'QSTOCK',$qst$);
endfor;

extract entite = select coalesce(max(Entite),0) from ENTITE;

for com,cli,dat = [select * from COMMANDE];
  compute entite = $entite$ + 1;

  insert into ENTITE values($entite$,'COMMANDE');
  insert into IDENTITE values($entite$,'COMMANDE','$com$');

  insert into ATTRIBUT values($entite$,'NCOM','$com$');
  insert into ATTRIBUT values($entite$,'DATECOM','$dat$');
endfor;

extract entite = select coalesce(max(Entite),0) from ENTITE;

for com,pro,qco = [select * from DETAIL];
  compute entite = $entite$ + 1;

  insert into ENTITE values($entite$,'DETAIL');
  insert into IDENTITE values($entite$,'DETAIL','$com$-$pro$');

  insert into ATTRIBUT values($entite$,'NCOM','$com$');
  insert into ATTRIBUT values($entite$,'NPRO','$pro$');
  insert into ATTRIBUT values($entite$,'QCOM',$qco$);
endfor;


-- ================================= --

for com,cli = [select I1.Entite,I2.Entite 
               from   COMMANDE M,IDENTITE I1,IDENTITE I2
               where  M.NCOM = I1.EID and I1.TypeE = 'COMMANDE'
               and    M.NCLI = I2.EID and I2.TypeE = 'CLIENT'];

  insert into ASSOCIATION values($com$,$cli$,'COMMANDE(NCLI)');
endfor;

for det,com,pro = [select DI.Entite,CI.Entite,PI.Entite
               from   DETAIL D,IDENTITE DI,IDENTITE CI,IDENTITE PI
               where  D.NCOM||'-'||D.NPRO = DI.EID and DI.TypeE = 'DETAIL'
               and    D.NCOM = CI.EID and CI.TypeE = 'COMMANDE'
               and    D.NPRO = PI.EID and PI.TypeE = 'PRODUIT'];

  insert into ASSOCIATION values($det$,$com$,'DETAIL(NCOM)');
  insert into ASSOCIATION values($det$,$pro$,'DETAIL(NPRO)');
endfor;

-- ================================= --

select * from ENTITE;
select * from IDENTITE;
select * from ATTRIBUT;
select * from ASSOCIATION order by Source,TypeA,Cible;
closeDB;
stop;


-- ---------------------------------- --
-- Vue exprimant les donnes sources --
-- ---------------------------------- --

create temporary view V_CLIENT as
select 
max(case when Attribut = 'NCLI'     then Valeur end) as NCLI,
max(case when Attribut = 'NOM'      then Valeur end) as NOM,
max(case when Attribut = 'ADRESSE'  then Valeur end) as ADRESSE,
max(case when Attribut = 'LOCALITE' then Valeur end) as LOCALITE,
max(case when Attribut = 'CAT'      then Valeur end) as CAT,
max(case when Attribut = 'COMPTE'   then cast(Valeur as real) else -999999.9 end) as COMPTE
from  CLICOM
where Entite in (select Entite from CLICOM 
                 where  Attribut = 'TypeE' and Valeur = 'CLIENT')
group by Entite;


write-b Table CLICOM ;
select * from CLICOM order by Entite;

write-b Table source CLIENT reconstitue;
select * from V_CLIENT;

closeDB;

stop;
