-- ----------------------------------- --
-- Bases de donnes non relationnelles --
-- Le modle de table universelle      --
-- ----------------------------------- --

openDB CLICOM.db;

create temporary table CLI_NOM(
   NCLI      char(10) not null primary key,
   NOM       char(32) not null);
create temporary table CLI_ADRESSE(
   NCLI      char(10) not null primary key,
   ADRESSE   char(60) not null);
create temporary table CLI_LOCALITE(
   NCLI      char(10) not null primary key,
   LOCALITE  char(30) not null);
create temporary table CLI_CAT(
   NCLI      char(10) not null primary key,
   CAT       char(2));
create temporary table CLI_COMPTE(
   NCLI      char(10) not null primary key,
   COMPTE    decimal(9,2) not null);

create temporary table PRO_LIBELLE(
   NPRO      char(15) not null primary key,
   LIBELLE   char(60) not null);
create temporary table PRO_PRIX(
   NPRO      char(15) not null primary key,
   PRIX    integer  not null);
create temporary table PRO_QSTOCK(
   NPRO      char(15) not null primary key,
   QSTOCK    integer  not null);

create temporary table COM_NCLI(
   NCOM      char(12) not null primary key,
   NCLI      char(10) not null);
create temporary table COM_DATECOM(
   NCOM      char(12) not null primary key,
   DATECOM   date     not null);

create temporary table DET_QCOM(
   NCOM      char(12) not null,
   NPRO      char(15) not null,
   QCOM      integer  not null,
   primary key (NCOM,NPRO));


-- ---------------------------------- --
-- Vue exprimant les donnes sources --
-- ---------------------------------- --

create temporary view V_CLIENT as
select  C1.NCLI,C1.NOM,C2.ADRESSE,C3.LOCALITE,C4.CAT,C5.COMPTE
from    CLI_NOM C1,CLI_ADRESSE C2,CLI_LOCALITE C3,
        CLI_CAT C4,CLI_COMPTE C5
where   C1.NCLI = C2.NCLI and C1.NCLI = C3.NCLI 
and     C1.NCLI = C4.NCLI and C1.NCLI = C5.NCLI;

create temporary view V_PRODUIT as
select  C1.NPRO,C1.LIBELLE,C2.PRIX,C3.QSTOCK
from    PRO_LIBELLE C1,PRO_PRIX C2,PRO_QSTOCK C3
where   C1.NPRO = C2.NPRO and C1.NPRO = C3.NPRO;

create temporary view V_COMMANDE as
select  C1.NCOM,C1.NCLI,C2.DATECOM
from    COM_NCLI C1,COM_DATECOM C2
where   C1.NCOM = C2.NCOM;

create temporary view V_DETAIL as
select  NCOM,NPRO,QCOM
from    DET_QCOM;


-- ------------------------ --
-- index d'identifiants     --
-- ------------------------ --

create unique index CLI_NCLI_NOM      on CLI_NOM(NCLI);
create unique index CLI_NCLI_ADRESSE  on CLI_ADRESSE(NCLI);
create unique index CLI_NCLI_LOCALITE on CLI_LOCALITE(NCLI);
create unique index CLI_NCLI_CAT      on CLI_CAT(NCLI);
create unique index CLI_NCLI_COMPTE   on CLI_COMPTE(NCLI);

create unique index PRO_NPRO_LIBELLE  on PRO_LIBELLE(NPRO);
create unique index PRO_NPRO_PRIX     on PRO_PRIX(NPRO);
create unique index PRO_NPRO_QSTOCK   on PRO_QSTOCK(NPRO);

create unique index COM_NCOM_NCLI     on COM_NCLI(NCOM);
create unique index COM_NCOM_DATECOM  on COM_DATECOM(NCOM);

create unique index det_NCOM_NPRO     on DET_QCOM(NCOM,NPRO);


-- ------------------------ --
-- index de cls trangres --
-- ------------------------ --

create index COM_NCLI_CLI on COM_NCLI(NCLI);
create index DET_NCOM_COM on DET_QCOM(NCOM);
create index DET_NPRO_PRO on DET_QCOM(NPRO);


-- ---------------------- --
-- Chargement des donnes --
-- ---------------------- --

insert into CLI_NOM      select NCLI,NOM       from CLIENT;
insert into CLI_ADRESSE  select NCLI,ADRESSE   from CLIENT;
insert into CLI_LOCALITE select NCLI,LOCALITE  from CLIENT;
insert into CLI_CAT      select NCLI,CAT       from CLIENT;
insert into CLI_COMPTE   select NCLI,COMPTE    from CLIENT;

insert into PRO_LIBELLE  select NPRO,LIBELLE   from PRODUIT;
insert into PRO_PRIX     select NPRO,PRIX      from PRODUIT;
insert into PRO_QSTOCK   select NPRO,QSTOCK   from PRODUIT;

insert into COM_NCLI     select NCOM,NCLI      from COMMANDE;
insert into COM_DATECOM  select NCOM,NCLI      from COMMANDE;

insert into DET_QCOM     select NCOM,NPRO,QCOM from DETAIL;



write-b Tables-colonnes de CLIENT;
select  * from CLI_NOM; 
select  * from CLI_ADRESSE; 
select  * from CLI_LOCALITE; 
select  * from CLI_CAT; 
select  * from CLI_COMPTE; 

write-b Tables-colonnes de PRODUIT;
select  * from PRO_LIBELLE; 
select  * from PRO_PRIX; 
select  * from PRO_QSTOCK; 

write-b Tables-colonnes de COMMANDE;
select  * from COM_NCLI; 
select  * from COM_DATECOM; 

write-b Tables-colonnes de DETAIL;
select  * from DET_QCOM; 

write-b Relecture des donnes sources;
select * from V_CLIENT order by NCLI;
select * from V_PRODUIT order by NPRO;
select * from V_COMMANDE order by NCOM;
select * from V_DETAIL order by NCOM,NPRO;

closeDB;

