
-- ------------------------------------------------------------ --
-- Cration de la base de donnes ABCDE-proto.db                --
-- Comporte des ID et FK complexes                              --
-- Utilise comme tude de cas pour la gnration de prototypes --
-- Son schma est document dans l'annexe A18                   --
-- ------------------------------------------------------------ --

createOrReplaceDB ABCDE-proto.db;

create table A(Nom varchar(10),IdA char(3),
               primary key (IdA));

create table B(Nom varchar(10),NomA varchar(10),IdB1 char(3) not null,IdB2 char(3) not null,RefA char(3),
               primary key (IdB1,IdB2),
               foreign key (RefA) references A);

create table C(Nom varchar(10),NomA varchar(20),IdC char(3) not null,RefA char(3),
               primary key (IdC),
               foreign key (RefA) references A);

create table D(Nom varchar(10),NomB1 varchar(10),NomB2 varchar(10),NomC varchar(10),
               IdD1 char(3) not null,IdD2 char(3) not null,IdD3 char(3) not null,
               RefB11  char(3),RefB12  char(3),
               RefB21  char(3),RefB22  char(3),
               RefC  char(3),
               primary key (IdD1,IdD2,IdD3),
               foreign key (RefB11,RefB12) references B,
               foreign key (RefB21,RefB22) references B,
               foreign key (RefC) references C);

create table E(Nom varchar(10),NomD varchar(10),NomE1 varchar(10),NomE2 varchar(10),
               IdE1 varchar(3) not null,IdE2 varchar(3) not null,
               RefD1 varchar(3),RefD2 varchar(3),RefD3 varchar(3),
               RefE11 varchar(3),RefE12 varchar(3),
               RefE21 varchar(3),RefE22 varchar(3),
               primary key (IdE1,IdE2),
               foreign key (RefD1,RefD2,REfD3) references D,
               foreign key (RefE11,RefE12) references E,
               foreign key (RefE21,RefE22) references E);

-- ------------------------- --
-- Remplissage de la table A --
-- ------------------------- --

insert into A values('Jean','a11');
insert into A values('Anne','a12');
insert into A values('Marie','a13');

write Table A;
select * from A;

-- ------------------------- --
-- Remplissage de la table B --
-- ------------------------- --

insert into B(Nom,NomA,IdB1,IdB2) values('Leon','Anne','b11','b21');
insert into B(Nom,NomA,IdB1,IdB2) values('Albert','Jean','b12','b22');
insert into B(Nom,NomA,IdB1,IdB2) values('Berthe','Jean','b13','b23');

-- Conversion de la cl trangre --
update B set RefA = (select IdA from A where Nom = B.NomA);

write Table B;
select * from B;

-- ------------------------- --
-- Remplissage de la table C --
-- ------------------------- --

insert into C(Nom,NomA,IdC) values('Yves','Marie','c11');
insert into C(Nom,NomA,IdC) values('Claire','Anne','c12');
insert into C(Nom,NomA,IdC) values('Loc','Marie','c13');

-- Conversion de la cl trangre --
update C set RefA = (select IdA from A where Nom = C.NomA);

write Table C;
select * from C;

-- ------------------------- --
-- Remplissage de la table D --
-- ------------------------- --

insert into D(Nom,NomB1,NomB2,NomC,IdD1,IdD2,IdD3) values('Pierre','Albert','Berthe','Yves', 'd11','d21','d31');
insert into D(Nom,NomB1,NomB2,NomC,IdD1,IdD2,IdD3) values('Julie', 'Albert','Leon','Claire', 'd12','d22','d32');
insert into D(Nom,NomB1,NomB2,NomC,IdD1,IdD2,IdD3) values('Manon', 'Berthe','Leon','Yves', 'd13','d23','d33');

-- Conversion des cls trangres --
update D set 
  RefB11 = (select IdB1 from B where Nom = D.NomB1), RefB12 = (select IdB2 from B where Nom = D.NomB1),
  RefB21 = (select IdB1 from B where Nom = D.NomB2), RefB22 = (select IdB2 from B where Nom = D.NomB2),
  RefC = (select IdC from C where Nom = D.NomC);

write Table D;
select * from D;

-- ------------------------- --
-- Remplissage de la table E --
-- ------------------------- --

insert into E(NOM,NOMD,NOME1,NOME2,iDE1,iDE2) values('Cedric', 'Pierre', 'Marc',  'Alice', 'e11','e21');
insert into E(NOM,NOMD,NOME1,NOME2,iDE1,iDE2) values('Marc',   'Manon',  'Alice', 'Leon', 'e12','e22');
insert into E(NOM,NOMD,NOME1,NOME2,iDE1,iDE2) values('Alice',  'Manon',  'Marc',  null,    'e13','e23');

-- Conversion des cls trangres --
update E set
   RefD1 = (select IdD1 from D where Nom = E.NomD),
   RefD2 = (select IdD2 from D where Nom = E.NomD),
   RefD3 = (select IdD3 from D where Nom = E.NomD),
   RefE11 = (select IdE1 from E EE where EE.Nom = E.NomE1),
   RefE12 = (select IdE2 from E EE where EE.Nom = E.NomE1),
   RefE21 = (select IdE1 from E EE where EE.Nom = E.NomE2),
   RefE22 = (select IdE2 from E EE where EE.Nom = E.NomE2);

write Table E;
select * from E;

commitDB;

closeDB;

stop;


-- Table A
-- +-------+-----+
-- | Nom   | IdA |
-- +-------+-----+
-- | Jean  | a11 |
-- | Anne  | a12 |
-- | Marie | a13 |
-- +-------+-----+
-- Table B
-- +--------+------+------+------+------+
-- | Nom    | NomA | IdB1 | IdB2 | RefA |
-- +--------+------+------+------+------+
-- | Leon   | Anne | b11  | b21  | a12  |
-- | Albert | Jean | b12  | b22  | a11  |
-- | Berthe | Jean | b13  | b23  | a11  |
-- +--------+------+------+------+------+
-- Table C
-- +--------+-------+-----+------+
-- | Nom    | NomA  | IdC | RefA |
-- +--------+-------+-----+------+
-- | Yves   | Marie | c11 | a13  |
-- | Claire | Anne  | c12 | a12  |
-- | Loc   | Marie | c13 | a13  |
-- +--------+-------+-----+------+
-- Table D
-- +--------+--------+--------+--------+------+------+------+--------+--------+--------+--------+------+
-- | Nom    | NomB1  | NomB2  | NomC   | IdD1 | IdD2 | IdD3 | RefB11 | RefB12 | RefB21 | RefB22 | RefC |
-- +--------+--------+--------+--------+------+------+------+--------+--------+--------+--------+------+
-- | Pierre | Albert | Berthe | Yves   | d11  | d21  | d31  | b12    | b22    | b13    | b23    | c11  |
-- | Julie  | Albert | Leon   | Claire | d12  | d22  | d32  | b12    | b22    | b11    | b21    | c12  |
-- | Manon  | Berthe | Leon   | Yves   | d13  | d23  | d33  | b13    | b23    | b13    | b23    | c11  |
-- +--------+--------+--------+--------+------+------+------+--------+--------+--------+--------+------+
-- Table E
-- +--------+--------+-------+-------+------+------+-------+-------+-------+--------+--------+--------+--------+
-- | Nom    | NomD   | NomE1 | NomE2 | IdE1 | IdE2 | RefD1 | RefD2 | RefD3 | RefE11 | RefE12 | RefE21 | RefE22 |
-- +--------+--------+-------+-------+------+------+-------+-------+-------+--------+--------+--------+--------+
-- | Cedric | Pierre | Marc  | Alice | e11  | e21  | d11   | d21   | d31   | e12    | e22    | e13    | e23    |
-- | Marc   | Manon  | Alice | Leon  | e12  | e22  | d13   | d23   | d33   | e13    | e23    | e13    | e23    |
-- | Alice  | Manon  | Marc  | --    | e13  | e23  | d13   | d23   | d33   | e12    | e22    | --     | --     |
-- +--------+--------+-------+-------+------+------+-------+-------+-------+--------+--------+--------+--------+


